@@ -32,6 +32,7 @@ from __future__ import (division, absolute_import, print_function,
 
 import logging
 import os
+import sys
 
 try:
     from urllib2 import Request, urlopen, HTTPError
@@ -98,7 +99,11 @@ def create_metainfo(path):
     if not os.path.isdir(path):
         return None
 
-    metainfo_file = os.path.join(path, 'metainfo.yaml')
+    try:
+        metainfo_file = os.path.join(path, 'metainfo.yaml')
+    except UnicodeDecodeError as e:
+        logging.warning('Unusual base path {!r} for metainfo.yaml'.format(path))
+        return None
     if not os.path.isfile(metainfo_file):
         return None
 
@@ -157,10 +162,11 @@ def parse_tree(rootdir):
     for path, dirs, _ in os.walk(rootdir):
         # We don't want to do the recursion in the dotdirs
         dirs[:] = [d for d in dirs if not d[0] == '.']
+        if sys.version_info.major < 3:
+            path = path.decode(sys.getfilesystemencoding())
         metainfo = create_metainfo(path)
         if metainfo is not None:
-
-            if metainfo['public_lib']:
+            if metainfo['public_lib'] or 'group_info' in metainfo:
                 metalist.append(metainfo)
             else:
                 logging.warning("{} has no public libraries"
@@ -185,7 +191,7 @@ def sort_metainfo(metalist, all_maintainers):
     products = dict()
     groups = []
     libraries = []
-    available_platforms = set(['Windows', 'MacOSX', 'Linux'])
+    available_platforms = set(['Windows', 'MacOSX', 'Linux', 'Android', 'FreeBSD'])
 
     # First extract the structural info
     for metainfo in metalist:
@@ -211,10 +217,17 @@ def sort_metainfo(metalist, all_maintainers):
 
         expand_platform_all(dct, available_platforms)
         platforms = dct
-        lib = Library(metainfo, products, platforms, all_maintainers)
-        libraries.append(lib)
 
-    groups = [p for p in list(products.values()) if len(p.libraries) > 1]
+        if metainfo['public_lib']:
+            lib = Library(metainfo, products, platforms, all_maintainers)
+            libraries.append(lib)
+
+    groups = []
+    for key in products.copy():
+        if len(products[key].libraries) == 0:
+            del products[key]
+        elif products[key].part_of_group:
+            groups.append(products[key])
 
     return list(products.values()), groups, libraries, available_platforms
 
@@ -236,8 +249,7 @@ def extract_product(metainfo, all_maintainers):
 
     try:
         product = Product(metainfo, all_maintainers)
+        return product
     except ValueError as e:
         logging.error(e)
-        product = None
-    finally:
-        return product
+        return None
